@@ -1,17 +1,19 @@
 from copy import deepcopy
+import logging
 import MDAnalysis as mda
 import numpy as np
 from openmm import *
 from openmm.app import *
 from openmm.unit import *
 from openmm.app.internal.singleton import Singleton
-import os
 from pathlib import Path
-from typing import Union
+from typing import Optional, Union
 
 PathLike = Union[Path, str]
 OptPath = Union[Path, str, None]
 
+logger = logging.getLogger(__name__)
+
 class Simulator:
     """
     Class for performing OpenMM simulations on AMBER FF inputs. Inputs must conform
@@ -19,17 +21,27 @@ class Simulator:
 
     Arguments:
         path (PathLike): Path to simulation inputs, same as output path.
-        equil_steps (int): Defaults to 1,250,000 (2.5 ns). Number of simulation timesteps to
-            perform equilibration for (2fs timestep).
-        prod_steps (int): Defaults to 250,000,000 (1 µs). Number of simulation timesteps to
-            perform production MD for (4fs timestep).
+        top_name (Optional[str]): Defaults to None. Optional topology file name. If not
+            provided, we will assume this is `system.prmtop`.
+        coor_name (Optional[str]): Defaults to None. Optional coordinate file name. If not
+            provided, we will assume this is `system.inpcrd`.
+        out_path (Optional[Path]): Defaults to None. Optional output path which is where we
+            will place the simulation outputs. If not provided, we will assume this is 
+            `self.path`.
+        ff (str): Switch for whether or not to utilize AMBER or CHARMM ff.
+        equil_steps (int): Defaults to 1,250,000 (2.5 ns at 2 fs timestep). Number of simulation 
+            timesteps to perform equilibration.
+        prod_steps (int): Defaults to 250,000,000 (1 µs at 4 fs timestep). Number of simulation 
+            timesteps to perform production MD.
         n_equil_cycles (int): Defaults to 3 cycles. Number of additional unrestrained 
             equilibration cycles to perform. Increasing this may triage unstable systems 
             at the cost of more simulation time albeit this may be negligible in the 
             grand scheme of things.
-        reporter_frequency (int): Defaults to 1,000 timesteps. How often to write out to
-            the logs and trajectory files in equilibration. X times less frequently during
-            production MD.
+        temperature (float): Defaults to 300.0. Temperature for simulation.
+        eq_reporter_frequency (int): Defaults to 1,000 timesteps. How often to write out to
+            the logs and trajectory files in equilibration.
+        prod_reporter_frequency (int): Defaults to 10,000 timesteps. How often to write out to
+            the logs and trajectory files in production.
         platform (str): Defaults to CUDA. Which OpenMM platform to simulate with (options
             include CUDA, CPU, OpenCL).
         device_ids (list[int]): Defaults to [0]. Which accelerators to use (multiple GPU 
@@ -38,36 +50,55 @@ class Simulator:
         force_constant (float): Defaults to 10.0 kcal/mol*Å^2. Force constant to use for 
             harmonic restraints during equilibration. Currently restraints are only applied
             to protein backbone atoms.
+        params (Optional[str]): Optional list of CHARMM parameter files for loading from 
+            psf/pdb file using CHARMM36m forcefield.
     """
     def __init__(self, 
                  path: PathLike, 
+                 top_name: Optional[str]=None,
+                 coor_name: Optional[str]=None,
+                 out_path: Optional[Path]=None,
+                 ff: str='amber',
                  equil_steps: int=1_250_000, 
                  prod_steps: int=250_000_000, 
                  n_equil_cycles: int=3,
-                 reporter_frequency: int=1_000,
+                 temperature: float=300.,
+                 eq_reporter_frequency: int=1_000,
+                 prod_reporter_frequency: int=10_000,
                  platform: str='CUDA',
                  device_ids: list[int]=[0],
-                 force_constant: float=10.):
+                 force_constant: float=10.,
+                 params: Optional[str]=None,
+                 membrane: bool=False):
         self.path = Path(path) # enforce path object
-        # input files
-        self.prmtop = path / 'system.prmtop'
-        self.inpcrd = path / 'system.inpcrd'
+        self.top_file = self.path / top_name if top_name is not None else self.path / 'system.prmtop'
+        self.coor_file = self.path / coor_name if coor_name is not None else self.path / 'system.inpcrd'
+        self.temperature = temperature
+
+        self.ff = ff.lower()
+        self.params = params # for charmm parameter sets
+        self.setup_barostat(membrane)
+
+        if out_path is not None:
+            p = out_path
+        else:
+            p = path
 
         # logging/checkpointing stuff
-        self.eq_state = path / 'eq.state'
-        self.eq_chkpt = path / 'eq.chk'
-        self.eq_log = path / 'eq.log'
-        self.eq_freq = reporter_frequency
+        self.eq_state = p / 'eq.state'
+        self.eq_chkpt = p / 'eq.chk'
+        self.eq_log = p / 'eq.log'
+        self.eq_dcd = p / 'eq.dcd'
+        self.eq_freq = eq_reporter_frequency
         
-        self.dcd = path / 'prod.dcd'
-        self.restart = path / 'prod.rst.chk'
-        self.state = path / 'prod.state'
-        self.chkpt = path / 'prod.chk'
-        self.prod_log = path / 'prod.log'
-        self.prod_freq = self.eq_freq * 10
+        self.dcd = p / 'prod.dcd'
+        self.restart = p / 'prod.rst.chk'
+        self.state = p / 'prod.state'
+        self.chkpt = p / 'prod.chk'
+        self.prod_log = p / 'prod.log'
+        self.prod_freq = prod_reporter_frequency
 
         # simulation details
-        self.indices = self.get_restraint_indices()
         self.equil_cycles = n_equil_cycles
         self.equil_steps = equil_steps
         self.prod_steps = prod_steps
@@ -78,33 +109,109 @@ class Simulator:
 
         if platform == 'CPU':
             self.properties = {}
+            
+    def setup_barostat(self,
+                       is_membrane_system: bool) -> None:
+        """Chooses the correct barostat for the system based on whether or not there is
+        a membrane present.
 
-    def load_amber_files(self) -> System:
+        Args:
+            is_membrane_system (bool): True if this is a membrane-containing system.
+            
+        Returns:
+            None
         """
-        Builds an OpenMM system using the prmtop/inpcrd files. PME is utilized for
+        self.barostat_args = {
+            'defaultPressure': 1 * bar,
+        }
+        
+        if is_membrane_system:
+            self.barostat = MonteCarloMembraneBarostat
+            self.barostat_args.update({
+                'defaultSurfaceTension': 0 * bar * nm,
+                'defaultTemperature': self.temperature * kelvin,
+                'xymode': MonteCarloMembraneBarostat.XYIsotropic,
+                'zmode': MonteCarloMembraneBarostat.ZFree
+            })
+        else:
+            self.barostat = MonteCarloBarostat
+            self.barostat_args.update({
+                'temperature': self.temperature * kelvin
+            })
+
+    def load_system(self) -> System:
+        """Loads correct set of files based on which forcefield we have specified.
+
+        Raises:
+            AttributeError: If a non-valid choice of forcefield is made (not amber or charmm).
+
+        Returns:
+            System: OpenMM system loaded from correct files/forcefield.
+        """
+        if self.ff == 'amber':
+            system = self.load_amber_files()
+        elif self.ff == 'charmm':
+            system = self.load_charmm_files()
+        else:
+            raise AttributeError(f'self.ff must be a valid MD forcefield [amber, charmm]!')
+
+        if not hasattr(self, 'indices'):
+            self.indices = self.get_restraint_indices()
+
+        return system
+        
+    def load_amber_files(self) -> System:
+        """Builds an OpenMM system using the prmtop/inpcrd files. PME is utilized for
         electrostatics and a 1 nm non-bonded cutoff is used as well as 1.5 amu HMR.
 
         Returns:
             (System): OpenMM system
         """
-        if isinstance(self.inpcrd, Path | str):
-            self.inpcrd = AmberInpcrdFile(str(self.inpcrd))
-            self.prmtop = AmberPrmtopFile(str(self.prmtop), 
-                                          periodicBoxVectors=self.inpcrd.boxVectors)
+        if not hasattr(self, 'coordinate'):
+            self.coordinate = AmberInpcrdFile(str(self.coor_file))
+            self.topology = AmberPrmtopFile(str(self.top_file), 
+                                            periodicBoxVectors=self.coordinate.boxVectors)
 
-        system = self.prmtop.createSystem(nonbondedMethod=PME,
-                                          removeCMMotion=False,
-                                          nonbondedCutoff=1. * nanometer,
-                                          constraints=HBonds,
-                                          hydrogenMass=1.5 * amu)
+        system = self.topology.createSystem(nonbondedMethod=PME,
+                                            removeCMMotion=False,
+                                            nonbondedCutoff=1. * nanometer,
+                                            constraints=HBonds,
+                                            hydrogenMass=1.5 * amu)
+
+        return system
     
+    def load_charmm_files(self) -> System:
+        """Builds an OpenMM system using the psf/pdb files. PME is utilized for 
+        electrostatics and a 1 nm non-bonded cutoff is used as well as 1.5 amu HMR.
+
+        Returns:
+            (System): OpenMM system
+        """
+        if not hasattr(self, 'coordinate'):
+            self.coordinate = PDBFile(str(self.coor_file))
+            self.topology = CharmmPsfFile(str(self.top_file), 
+                                          periodicBoxVectors=self.coordinate.topology.getPeriodicBoxVectors())
+        if not hasattr(self, parameter_set) and self.params is not None:
+            self.parameter_set = CharmmParameterSet(*self.params)
+        
+        if self.params is None:
+            self.forcefield = ForceField('charmm36_2024.xml', 'charmm36/water.xml')
+            system = self.forcefield.createSystem(self.coordinate.topology,
+                                                  nonbondedMethod=PME,
+                                                  nonbondedCutoff=1.2 * nanometer,
+                                                  constraints=HBonds)
+        else:
+            system = self.topology.createSystem(self.parameter_set, 
+                                                nonbondedMethod=PME, 
+                                                nonbondedCutoff=1.2 * nanometer,
+                                                constraints=HBonds)
+        
         return system
     
     def setup_sim(self, 
                   system: System, 
                   dt: float) -> tuple[Simulation, Integrator]:
-        """
-        Builds OpenMM Integrator and Simulation objects utilizing a provided
+        """Builds OpenMM Integrator and Simulation objects utilizing a provided
         OpenMM System object and integration timestep, dt.
 
         Arguments:
@@ -115,10 +222,10 @@ class Simulator:
             (tuple[Simulation, Integrator]): A tuple containing both the Simulation
                 and Integrator objects.
         """
-        integrator = LangevinMiddleIntegrator(300*kelvin, 
+        integrator = LangevinMiddleIntegrator(self.temperature*kelvin, 
                                               1/picosecond, 
                                               dt*picoseconds)
-        simulation = Simulation(self.prmtop.topology, 
+        simulation = Simulation(self.topology.topology, 
                                 system, 
                                 integrator, 
                                 self.platform, 
@@ -137,16 +244,22 @@ class Simulator:
         skip_eq = all([f.exists() 
                        for f in [self.eq_state, self.eq_chkpt, self.eq_log]])
         if not skip_eq:
+            logger.info('No restart detected, will begin equilibration.')
             self.equilibrate()
+            logger.info(f'Equilibration finished, running {self.prod_steps} steps of production MD.')
 
         if self.restart.exists():
+            logger.info('Checkpoint file detected, resuming simulation.')
             self.check_num_steps_left()
-            self.production(chkpt=self.restart, 
+            logger.info(f'Will run {self.prod_steps} steps of production MD.')
+            self.production(chkpt=str(self.restart), 
                             restart=True)
         else:
-            self.production(chkpt=self.eq_chkpt,
+            self.production(chkpt=str(self.eq_chkpt),
                             restart=False)
 
+        logger.info('Production MD run complete.')
+
     def equilibrate(self) -> Simulation:
         """
         Sets up and runs equilibrium MD, including energy minimization to begin.
@@ -157,24 +270,25 @@ class Simulator:
         Returns:
             (Simulation): OpenMM simulation object.
         """
-        system = self.add_backbone_posres(self.load_amber_files(), 
-                                          self.inpcrd.positions, 
-                                          self.prmtop.topology.atoms(), 
+        system = self.load_system()
+        system = self.add_backbone_posres(system, 
+                                          self.coordinate.positions, 
+                                          self.topology.topology.atoms(), 
                                           self.indices,
                                           self.k)
     
         simulation, integrator = self.setup_sim(system, dt=0.002)
         
-        simulation.context.setPositions(self.inpcrd.positions)
+        simulation.context.setPositions(self.coordinate.positions)
         simulation.minimizeEnergy()
         
         simulation.reporters.append(StateDataReporter(str(self.eq_log), 
-                                                      str(self.eq_freq), 
+                                                      self.eq_freq, 
                                                       step=True,
                                                       potentialEnergy=True,
                                                       speed=True,
                                                       temperature=True))
-        simulation.reporters.append(DCDReporter(str(self.path / 'eq.dcd'), self.eq_freq))
+        simulation.reporters.append(DCDReporter(str(self.eq_dcd), self.eq_freq))
 
         simulation, integrator = self._heating(simulation, integrator)
         simulation = self._equilibrate(simulation)
@@ -195,10 +309,10 @@ class Simulator:
             restart (bool): Defaults to False. Flag to ensure we log the full simulation
                 to reporter log. Otherwise restarts will overwrite the original log file.
         """
-        system = self.load_amber_files()
-        simulation, integrator = self.setup_sim(system, dt=0.004)
+        system = self.load_system()
+        simulation, _ = self.setup_sim(system, dt=0.004)
         
-        system.addForce(MonteCarloBarostat(1*atmosphere, 300*kelvin))
+        system.addForce(self.barostat(*self.barostat_args.values()))
         simulation.context.reinitialize(True)
 
         if restart:
@@ -210,10 +324,10 @@ class Simulator:
         simulation = self.attach_reporters(simulation,
                                            self.dcd,
                                            log_file,
-                                           self.restart,
+                                           str(self.restart),
                                            restart=restart)
     
-        self._production(simulation)
+        self.simulation = self._production(simulation) # save simulation object
     
     def load_checkpoint(self, 
                         simulation: Simulation, 
@@ -311,14 +425,15 @@ class Simulator:
         
         integrator.setTemperature(T * kelvin)
         mdsteps = 100000
-        length = mdsteps // 1000
-        tstep = (300 - T) / length
+        heat_steps = 1000
+        length = mdsteps // heat_steps
+        tstep = (self.temperature - T) / length
         for i in range(length):
-          simulation.step(mdsteps//60)
+          simulation.step(heat_steps)
           temp = T + tstep * (1 + i)
           
-          if temp > 300:
-            temp = 300
+          if temp > self.temperature:
+            temp = self.temperature
           
           integrator.setTemperature(temp * kelvin)
     
@@ -356,7 +471,7 @@ class Simulator:
         simulation.context.setParameter('k', 0)
         simulation.step(eq_steps)
     
-        simulation.system.addForce(MonteCarloBarostat(1*atmosphere, 300*kelvin))
+        simulation.system.addForce(self.barostat(*self.barostat_args.values()))
         simulation.step(self.equil_cycles * eq_steps)
 
         simulation.saveState(str(self.eq_state))
@@ -398,7 +513,7 @@ class Simulator:
         Returns:
             (list[int]): List of atomic indices for atoms to be restrained.
         """
-        u = mda.Universe(str(self.prmtop), str(self.inpcrd))
+        u = mda.Universe(str(self.top_file), str(self.coor_file))
         if addtl_selection:
             sel = u.select_atoms(f'backbone or nucleicbackbone or {addtl_selection}')
         else:
@@ -493,17 +608,27 @@ class ImplicitSimulator(Simulator):
     
     Arguments:
         path (PathLike): Path to simulation inputs, same as output path.
-        equil_steps (int): Defaults to 1,250,000 (2.5 ns). Number of simulation timesteps to
-            perform equilibration for (2fs timestep).
-        prod_steps (int): Defaults to 250,000,000 (1 µs). Number of simulation timesteps to
-            perform production MD for (4fs timestep).
+        top_name (Optional[str]): Defaults to None. Optional topology file name. If not
+            provided, we will assume this is `system.prmtop`.
+        coor_name (Optional[str]): Defaults to None. Optional coordinate file name. If not
+            provided, we will assume this is `system.inpcrd`.
+        out_path (Optional[Path]): Defaults to None. Optional output path which is where we
+            will place the simulation outputs. If not provided, we will assume this is 
+            `self.path`.
+        ff (str): Switch for whether or not to utilize AMBER or CHARMM ff.
+        equil_steps (int): Defaults to 1,250,000 (2.5 ns at 2 fs timestep). Number of simulation 
+            timesteps to perform equilibration.
+        prod_steps (int): Defaults to 250,000,000 (1 µs at 4 fs timestep). Number of simulation 
+            timesteps to perform production MD.
         n_equil_cycles (int): Defaults to 3 cycles. Number of additional unrestrained 
             equilibration cycles to perform. Increasing this may triage unstable systems 
             at the cost of more simulation time albeit this may be negligible in the 
             grand scheme of things.
-        reporter_frequency (int): Defaults to 1,000 timesteps. How often to write out to
-            the logs and trajectory files in equilibration. X times less frequently during
-            production MD.
+        temperature (float): Defaults to 300.0. Temperature for simulation.
+        eq_reporter_frequency (int): Defaults to 1,000 timesteps. How often to write out to
+            the logs and trajectory files in equilibration.
+        prod_reporter_frequency (int): Defaults to 10,000 timesteps. How often to write out to
+            the logs and trajectory files in production.
         platform (str): Defaults to CUDA. Which OpenMM platform to simulate with (options
             include CUDA, CPU, OpenCL).
         device_ids (list[int]): Defaults to [0]. Which accelerators to use (multiple GPU 
@@ -518,19 +643,33 @@ class ImplicitSimulator(Simulator):
     """
     def __init__(self, 
                  path: str, 
+                 top_name: Optional[str]=None,
+                 coor_name: Optional[str]=None,
+                 out_path: Optional[Path]=None,
+                 ff: str = 'amber',
                  equil_steps: int=1_250_000, 
                  prod_steps: int=250_000_000, 
                  n_equil_cycles: int=3,
-                 reporter_frequency: int=1_000,
+                 temperature: float=300.,
+                 eq_reporter_frequency: int=1_000,
+                 prod_reporter_frequency: int=10_000,
                  platform: str='CUDA',
                  device_ids: list[int]=[0],
                  force_constant: float=10.,
                  implicit_solvent: Singleton=GBn2,
                  solute_dielectric: float=1.,
-                 solvent_dielectric: float=78.5):
-        super().__init__(path, equil_steps, prod_steps, n_equil_cycles,
-                         reporter_frequency, platform, device_ids, 
-                         force_constant)
+                 solvent_dielectric: float=78.5,
+                 **kwargs):
+        super().__init__(path=path, top_name=top_name, 
+                         coor_name=coor_name, out_path=out_path,
+                         ff=ff, equil_steps=equil_steps, 
+                         prod_steps=prod_steps, 
+                         n_equil_cycles=n_equil_cycles,
+                         temperature=temperature,
+                         eq_reporter_frequency=eq_reporter_frequency, 
+                         prod_reporter_frequency=prod_reporter_frequency,
+                         platform=platform, device_ids=device_ids, 
+                         force_constant=force_constant)
         self.solvent = implicit_solvent
         self.solute_dielectric = solute_dielectric
         self.solvent_dielectric = solvent_dielectric
@@ -545,18 +684,18 @@ class ImplicitSimulator(Simulator):
         Returns:
             (System): OpenMM system object.
         """
-        if isinstance(self.inpcrd, Path | str):
-            self.inpcrd = AmberInpcrdFile(self.inpcrd)
-            self.prmtop = AmberPrmtopFile(self.prmtop)
+        if not hasattr(self, 'coordinate'):
+            self.coordinate = AmberInpcrdFile(str(self.coor_file))
+            self.topology = AmberPrmtopFile(str(self.top_file))
 
-        system = self.prmtop.createSystem(nonbondedMethod=NoCutoff,
-                                          removeCMMotion=False,
-                                          constraints=HBonds,
-                                          hydrogenMass=1.5 * amu,
-                                          implicitSolvent=self.solvent,
-                                          soluteDielectric=self.solute_dielectric,
-                                          solventDielectric=self.solvent_dielectric,
-                                          implicitSolventKappa=self.kappa/nanometer)
+        system = self.topology.createSystem(nonbondedMethod=NoCutoff,
+                                            removeCMMotion=False,
+                                            constraints=HBonds,
+                                            hydrogenMass=1.5 * amu,
+                                            implicitSolvent=self.solvent,
+                                            soluteDielectric=self.solute_dielectric,
+                                            solventDielectric=self.solvent_dielectric,
+                                            implicitSolventKappa=self.kappa/nanometer)
     
         return system
         
@@ -569,33 +708,67 @@ class ImplicitSimulator(Simulator):
         Returns:
             (Simulation): OpenMM simulation object.
         """
-        system = self.add_backbone_posres(self.load_amber_files(), 
-                                          self.inpcrd.positions, 
-                                          self.prmtop.topology.atoms(), 
+        system = self.load_system()
+        system = self.add_backbone_posres(system, 
+                                          self.coordinate.positions, 
+                                          self.topology.topology.atoms(), 
                                           self.indices,
                                           self.k)
     
         simulation, integrator = self.setup_sim(system, dt=0.002)
         
-        simulation.context.setPositions(self.inpcrd.positions)
+        simulation.context.setPositions(self.coordinate.positions)
         state = simulation.context.getState(getEnergy=True)
         print(f'Energy before minimization: {state.getPotentialEnergy()}')
         simulation.minimizeEnergy()
         state = simulation.context.getState(getEnergy=True)
         print(f'Energy after minimization: {state.getPotentialEnergy()}')
         
-        simulation.reporters.append(StateDataReporter(self.eq_log, 
+        simulation.reporters.append(StateDataReporter(str(self.eq_log), 
                                                       self.eq_freq, 
                                                       step=True,
                                                       potentialEnergy=True,
                                                       speed=True,
                                                       temperature=True))
-        simulation.reporters.append(DCDReporter(f'{self.path}/eq.dcd', self.eq_freq))
+        simulation.reporters.append(DCDReporter(str(self.eq_dcd), self.eq_freq))
 
         simulation, integrator = self._heating(simulation, integrator)
         simulation = self._equilibrate(simulation)
         
         return simulation
+    
+    def production(self, 
+                   chkpt: PathLike, 
+                   restart: bool=False) -> None:
+        """
+        Performs production MD. Loads a new system, integrator and simulation object
+        and loads equilibration or past production checkpoint to allow either the
+        continuation of a past simulation or to inherit the outputs of equilibration.
+
+        Arguments:
+            chkpt (PathLike): The checkpoint file to load. Should be either the equilibration
+                checkpoint or a past production checkpoint.
+            restart (bool): Defaults to False. Flag to ensure we log the full simulation
+                to reporter log. Otherwise restarts will overwrite the original log file.
+        """
+        system = self.load_system()
+        simulation, _ = self.setup_sim(system, dt=0.004)
+        
+        simulation.context.reinitialize(True)
+
+        if restart:
+            log_file = open(str(self.prod_log), 'a')
+        else:
+            log_file = str(self.prod_log)
+
+        simulation = self.load_checkpoint(simulation, chkpt)
+        simulation = self.attach_reporters(simulation,
+                                           self.dcd,
+                                           log_file,
+                                           str(self.restart),
+                                           restart=restart)
+    
+        self.simulation = self._production(simulation) # save simulation object
 
 class CustomForcesSimulator(Simulator):
     """
@@ -647,15 +820,18 @@ class CustomForcesSimulator(Simulator):
         Returns:
             (System): OpenMM system object with custom forces.
         """
-        if isinstance(self.inpcrd, str):
-            self.inpcrd = AmberInpcrdFile(self.inpcrd)
-            self.prmtop = AmberPrmtopFile(self.prmtop, periodicBoxVectors=self.inpcrd.boxVectors)
+        if not hasattr(self, 'coordinate'):
+            self.coor_file = self.path / 'system.inpcrd'
+            self.top_file = self.path / 'system.prmtop'
+            self.coordinate = AmberInpcrdFile(str(self.coor_file))
+            self.topology = AmberPrmtopFile(str(self.top_file), 
+                                            periodicBoxVectors=self.coordinate.boxVectors)
 
-        system = self.prmtop.createSystem(nonbondedMethod=PME,
-                                          removeCMMotion=False,
-                                          nonbondedCutoff=1. * nanometer,
-                                          constraints=HBonds,
-                                          hydrogenMass=1.5 * amu)
+        system = self.topology.createSystem(nonbondedMethod=PME,
+                                            removeCMMotion=False,
+                                            nonbondedCutoff=1. * nanometer,
+                                            constraints=HBonds,
+                                            hydrogenMass=1.5 * amu)
 
         system = self.add_forces(system)
 
@@ -693,19 +869,20 @@ class Minimizer:
     """
     def __init__(self,
                  topology: PathLike,
-                 coordinates: OptPath = None,
+                 coordinates: PathLike,
                  out: PathLike='min.pdb',
                  platform: str='OpenCL',
-                 device_ids: list[int]=[0]):
+                 device_ids: list[int] | None =[0]):
         self.topology = Path(topology) 
-        if coordinates is not None:
-            self.coordinates = Path(coordinates)
+        self.coordinates = Path(coordinates)
 
         self.path = self.topology.parent
         self.out = self.path / out
-        self.platform = platform
-        self.properties = {'DeviceIndex': ','.join([str(x) for x in device_ids]),
-                           'Precision': 'mixed'}
+        self.platform = Platform.getPlatformByName(platform)
+        self.properties = {'Precision': 'mixed'}
+
+        if device_ids is not None:
+            self.properties.update({'DeviceIndex': ','.join([str(x) for x in device_ids])})
 
     def minimize(self) -> None:
         """
@@ -732,7 +909,7 @@ class Minimizer:
         state = simulation.context.getState(getPositions=True)
         positions = state.getPositions()
         
-        PDBFile.writeFile(simulation.topology, 
+        PDBFile.writeFile(self.topology.topology, 
                           positions, 
                           file=str(self.out), 
                           keepIds=True)
@@ -764,12 +941,9 @@ class Minimizer:
         Returns:
             (System): OpenMM system object.
         """
-        inpcrd = list(self.path.glob('*.inpcrd')) + list(self.path.glob('*.rst7'))
-        if hasattr(self, 'coordinates'):
-            self.coordinates = AmberInpcrdFile(str(self.coordinates))
-        else:
-            self.coordinates = AmberInpcrdFile(str(inpcrd[0]))
-        self.topology = AmberPrmtopFile(str(self.topology))
+        self.coordinates = AmberInpcrdFile(str(self.coordinates))
+        self.topology = AmberPrmtopFile(str(self.topology), 
+                                        periodicBoxVectors=self.coordinates.boxVectors)
 
         system = self.topology.createSystem(nonbondedMethod=NoCutoff,
                                             constraints=HBonds)
